@@ -1,8 +1,8 @@
 /* gdb_interface.c - core analysis suite
  *
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002-2013 David Anderson
- * Copyright (C) 2002-2013 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002-2015 David Anderson
+ * Copyright (C) 2002-2015 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -592,6 +592,9 @@ gdb_command_string(int cmd, char *buf, int live)
         case GNU_SET_CRASH_BLOCK:
                 sprintf(buf, "GNU_SET_CRASH_BLOCK");
 		break;
+	case GNU_GET_FUNCTION_RANGE:
+                sprintf(buf, "GNU_GET_FUNCTION_RANGE");
+                break;
 	case 0:
 		buf[0] = NULLCHAR;
 		break;
@@ -788,7 +791,7 @@ cmd_gdb(void)
 	 */
 	if (!is_restricted_command(*argv, FAULT_ON_ERROR)) {
 		if (STREQ(pc->command_line, "gdb")) {
-			strcpy(buf, &pc->orig_line[3]);
+			strcpy(buf, first_space(pc->orig_line));
 			strip_beginning_whitespace(buf);
 		} else
 			strcpy(buf, pc->orig_line);
@@ -812,6 +815,7 @@ gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 	char locbuf[SIZEOF_32BIT], *p1;
 	uint32_t *p2;
 	int memtype;
+	ulong readflags;
 
 	if (write)
 		return FALSE;
@@ -819,6 +823,9 @@ gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 	if (pc->cur_req->flags & GNU_NO_READMEM)
 		return TRUE;
 
+	readflags = pc->curcmd_flags & PARTIAL_READ_OK ?
+		RETURN_ON_ERROR|RETURN_PARTIAL : RETURN_ON_ERROR;
+
 	if (pc->curcmd_flags & MEMTYPE_UVADDR)
 		memtype = UVADDR;
 	else if (pc->curcmd_flags & MEMTYPE_FILEADDR)
@@ -842,7 +849,7 @@ gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 
 	if (memtype == FILEADDR)
 		return(readmem(pc->curcmd_private, memtype, buf, len,
-                	"gdb_readmem_callback", RETURN_ON_ERROR));
+			"gdb_readmem_callback", readflags));
 	
 	switch (len)
 	{
@@ -853,7 +860,7 @@ gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 			return TRUE;
 
 		if (!readmem(addr, memtype, locbuf, SIZEOF_32BIT,
-		    "gdb_readmem_callback", RETURN_ON_ERROR)) 
+		    "gdb_readmem_callback", readflags)) 
 			return FALSE;
 
 		*p1 = locbuf[0];
@@ -868,7 +875,7 @@ gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 			return TRUE;
 
 		if (!readmem(addr, memtype, buf, SIZEOF_32BIT, 
-		    "gdb_readmem callback", RETURN_ON_ERROR))
+		    "gdb_readmem callback", readflags))
 			return FALSE;
 
 		if (memtype == KVADDR)
@@ -877,9 +884,8 @@ gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 		return TRUE;
 	}
 
-	return(readmem(addr, memtype, buf, len,
-                "gdb_readmem_callback", RETURN_ON_ERROR));
-
+	return(readmem(addr, memtype, buf, len, 
+		"gdb_readmem_callback", readflags));
 }
 
 /*
